# GCSI, type II
# beta: fixed value - K, E, a, b, Q1:  LM regression
# including calculation of LATERAL-VERTICAL interaction
# for mono-layer detection
# import of experimental data from excel file
# export of the results by appending to the input file

# load additional packages:
library(readxl)
library(xlsx)
library(minpack.lm)

# DATA INPUT (experimental values + estimated start parameters)
# use an excel file according to the appended template 
file <- file.choose()
adsorption <- read_excel(file, sheet = 1)
xall <- as.numeric(adsorption[[1]])
xall <- xall[!xall%in% c(NA)]   # removing NA statements
yall <- as.numeric(adsorption[[2]])
yall <- yall[!yall%in% c(NA)]   # removing NA statements
print(xall)
print(yall)
param.names <- adsorption[[3]]
param <- adsorption[[4]]
Nx <- length(yall) # alternatively you may define lower values of Nx
print(Nx) 
# elements used for calculation
x <- xall[1:Nx]
y <- yall[1:Nx]
w <- 1/y # statistical weight
# start parameters
K1s <- param[[1]]
Ks <-  param[[2]]
as <-  param[[3]]
bs <-  param[[4]]
Q1s<-  param[[5]]
E1s <- log(K1s/Ks) # E1 is used as adjusting parameter instead of K1
print(Ks)
print(E1s)
print(as)
print(bs)
print(Q1s)
beta <- 1000 # maximum number of clustering 

# lower and upper borders
K_1 <- Ks-0.2      
K_2 <- Ks+0.2

E1_1<- E1s-6
E1_2<- E1s+3

a_1 <- -6
a_2 <-  1

b_1 <- -1
b_2 <- -0

Q1_1 <- 0.01
Q1_2 <- 3


r  <- c(length=beta)
Ki <- c(length=beta)
K  <- Ks
E1 <- E1s
a  <- as
b  <- bs
Q1 <- Q1s
model <- function(K, E1, a, b, Q1){
Ki[1] <- K*exp(E1)
r <- 2L
while (r <= beta){
  Ev <- 1/3*E1*r^(a+b*r)
  El <- 2/3*E1*r^(b*r)
  E  <- Ev+El
  Ki[r] <- K*exp(E)
  r <- r + 1L
}
Ci <- c(length=beta)
Ci[1] <- Ki[1]/K
s <- 2L
while (s <= beta){
  Ci[s] <- Ci[s-1]*Ki[s]/K
  s <- s+1L
}
q <- K*x
# calculation of A, B, and Y
A <- c(length=beta)
A <- Ci[1]*q
j <- 2L
while (j <= beta){
  A <- Ci[j]*j*q^j + A
  j <- j + 1L
}
B <- c(length=beta)
B <- (1 + Ci[1]*q)
j <- 2L
while (j <= beta){
  B <- Ci[j]*q^j + B
  j <- j + 1L
}
Y <- Q1*A/B
return(Y)
} # end of the function subroutine

Ym <- model(K, E1, a, b, Q1)

YY <- nlsLM(y ~ model(K, E1, a, b, Q1), start = list (K = Ks, E1 = E1s, a = as, b = bs, Q1 = Q1s), 
            lower= c(K_1, E1_1, a_1, b_1, Q1_1), upper= c(K_2, E1_2, a_2, b_2, Q1_2), weights = w, control = list(maxiter=1000, ftol =1e-10),trace = TRUE)

fitted(YY)  # optimized values
resid(YY)   # residuals
delta <- y - mean(y)
Sm <- sum(resid(YY)^2)
Rregr <- (1 - Sm/sum(delta^2))^0.5 # correlation coefficient
SS <- sum(w*resid(YY)^2)
print(SS)   # weighted residual sum of squares
Np <- 6     # number of adjustable parameters (including sum of squares)
AICc <- Nx*log(SS/Nx) + 2*Np + 2*Np*(Np+1)/(Nx-Np-1) # Akaike criterion
print(Sm)   # non-weighted statistical residual sum of squares
K <-  coef(YY)[[1]]
E1 <- coef(YY)[[2]]
a <-  coef(YY)[[3]]
b <-  coef(YY)[[4]]
Q1<-  coef(YY)[[5]]
K1 <- K*exp(E1)
K1_1<-K*exp(E1_1)
K1_2<-K*exp(E1_2)
coef.err <- summary(YY)$coefficients[,2]
K. <- coef.err[[1]]  
E1. <-coef.err[[2]] 
a. <- coef.err[[3]] 
b. <- coef.err[[4]] 
Q1.<- coef.err[[5]] 
K1. <-K1*E1.
print(AICc)
print(Rregr)
summary(YY)
print(beta)
print(coef(YY)) # optimized parameters
print(coef.err) # standard error of parameters

if(K <= K_1){
  print("K restricted by lower limit")
}
if(K >= K_2){
  print("K restricted by upper limit")
}
if(E1 <= E1_1){
  print("E1 restricted by lower limit")
}
if(E1 >= E1_2){
  print("E1 restricted by upper limit")
}
if(a <= a_1){
  print( "a restricted by lower limit")
}
if(a >= a_2){
  print( "a restricted by upper limit")
}
if(b <= b_1){
  print(" b restricted by lower limit")
}
if(b >= b_2){
  print(" b restricted by upper limit")
}
if(Q1 <= Q1_1){
  print("Q1 restricted by lower limit")
}
if(Q1 >= Q1_2){
  print("Q1 restricted by upper limit")
}

# SIMULATION ------------------------------------------------------------------
# row of x values from 1E-7 to 1.0
N <- 700
N2 <- 1000
log_qdata <- -(N-1):-130
qdata1 <- 10^(log_qdata/100)
qdata2 <- ((0.051*N2):N2)/N2
qdata  <- c(qdata1,qdata2)
xdata. <- qdata/Ks
Nxdata.<- length(xdata.)
om <-  0                          # these last entries are omitted
xdata <- xdata.[1:(Nxdata.- om)]  # to avoid miscalculation of derivation

n <-   c(length=beta)
Kn <-  c(length=beta)
Kv1n <-c(length=beta)
Kl1n <-c(length=beta)
Cn  <- c(length=beta)
Cv1n <-c(length=beta)
Cl1n <-c(length=beta)
Ev1n <-c(length=beta)
El1n <-c(length=beta)
Ev1 <- E1*2^(a+b*2)
El1 <- E1*2^(b*2)
n[1] <- 1
Kn[1]  <- K*exp(E1)
Kv1n[1] <-K*exp(Ev1*1/3)
Kl1n[1] <-K*exp(El1*2/3)

r <- 2L
while (r <= beta){
  n[r] <- r
  Ev <-   1/3*E1*r^(a+b*r)           # vertical energy 
  Ev1n <- 1/3*E1*(r+1)^(a+b*(r+1)) # # vertical energy +1
  El <-   2/3*E1*r^(b*r)             # lateral energy
  El1n <- 2/3*E1*(r+1)^(b*(r+1))   # # lateral energy +1
  E <- Ev+El
  Kv1n[r]<- K*exp(Ev1n) 
  Kl1n[r]<- K*exp(El1n) 
  Kn[r]  <- K*exp(E)
  r <- r + 1L
}
Cn[1]  <- Kn[1]/K
Cv1n[1] <-Kv1n[1]/K 
Cl1n[1] <-Kl1n[1]/K
s <- 2L
while (s <= beta){
  Cn[s]  <- Cn[s-1]*Kn[s]/K
  Cv1n[s] <-Cv1n[s-1]*Kv1n[s]/K
  Cl1n[s] <-Cl1n[s-1]*Kl1n[s]/K
  s <- s+1L
}
# calculation of A, B, and simulated Ys
q <-   K*xdata
A <-   Cn[1]*q
Av1 <- Cv1n[1]*q
Al1 <- Cl1n[1]*q
j <- 2L
while (j <= beta){
  A <- Cn[j]*j*q^j + A
  Av1<-Cv1n[j]*j*q^j + Av1
  Al1<-Cl1n[j]*j*q^j + Al1
  j <- j + 1L
}
B <- (1 + Cn[1]*q)
Bv1<-(1 + Cv1n[1]*q) 
Bl1<-(1 + Cl1n[1]*q) 
j <- 2L
while (j <= beta){
  B  <- Cn[j]*q^j + B
  Bv1<- Cv1n[j]*q^j + Bv1
  Bl1<- Cl1n[j]*q^j + Bl1
  j <- j + 1L
}
Ys   <- Q1*A/B
Yv1s <- Q1*Av1/Bv1
Yl1s <- Q1*Al1/Bl1
delta_v  <- (Ys-Yv1s)/Ys
delta_l  <- (Ys-Yl1s)/Ys
# ratio of differential vertical to differential lateral interaction
vl <- delta_v/delta_l 
# print(vl)

# monolayer condition ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# derivation of vl against adsorbed amount Ys

Nxdata <- length(xdata)
p. <- c(length=Nxdata)
m <- 3L
while (m <= Nxdata-1){  
  vl. <- vl[m]-vl[m-1]
  Ys. <- Ys[m]-Ys[m-1]
  p.[m]  <- vl./Ys.
  m <- m + 1L
}
pp. <- p.[3:(Nxdata-1)]
# print(pp.)
print(max(pp.))
mono <-  which(pp.==max(pp.), arr.ind=TRUE)
print(mono)
print(xdata[mono])
Ys1 <- Ys[mono]
print(Ys1)

#*****************************************************************************
# data export  -----------------------------------------------------------------
# output, sheet name: "isotherm", appending to the same excel file as for input
pressure <- c(xdata)
adsorbed <- c(Ys)
vertical <- delta_v 
lateral  <- delta_l
ver_lat  <- vl
derivative <- c("nd", "nd",  pp., "nd")
output1 <- data.frame(pressure,adsorbed,vertical,lateral,ver_lat,derivative)
write.xlsx(output1, file, sheetName = "Isotherm", col.names = TRUE, row.names = FALSE, append = TRUE)

# output excel file, sheet name: "energy relationship"
index.n <- n
Kn <- c(Kn)
Cn <- c(Cn)
output2 <- data.frame(index.n,Kn,Cn)
write.xlsx(output2, file, sheetName = "energy rel", col.names = TRUE, row.names = FALSE, append = TRUE)

# output excel file, sheet name: "parameters"
name <- c("K1=","E1  =","K  =","a  =","b  =","Q1=","R  =","AICc =", "monolayer")
parameters <-  c(K1,E1,K,a,b,Q1,Rregr,AICc,Ys1)
errors <- c(K1.,E1., K., a., b., Q1., "nd", "nd","nd")
low.lim <- c(K1_1,E1_1,K_1,a_1,b_1,Q1_1,"nd","nd","nd")
upper.lim <- c(K1_2,E1_2,K_2,a_2,b_2,Q1_2,"nd","nd","nd")
output3 <- data.frame(name, parameters, errors, low.lim, upper.lim)
write.xlsx(output3, file, sheetName = "parameters", col.names = TRUE, row.names = FALSE, append = TRUE)

#end